//
//  DummyView.m
//  Undo Inspector
//
//  Created by Patrick Geiller on 19/06/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "DummyView.h"
#import "UndoWindow.h"


@implementation DummyView

//
// Allocate our objects array
//
- (void)awakeFromNib
{
	objects = [[NSMutableArray alloc] init];
	quadSize = 50;
}

//
// Draw rects, draw selected rect with a lighter color
//
- (void)drawRect:(NSRect)rect {
    // Drawing code here.

	[[NSColor colorWithCalibratedWhite:0.2 alpha:1] set];
	[[NSBezierPath bezierPathWithRect:rect] fill];
	
	for (DummyObject* object in objects)
	{
		NSPoint p = [object position];
		NSBezierPath* path = [NSBezierPath bezierPathWithRoundedRect:NSMakeRect(p.x-quadSize/2, p.y-quadSize/2, quadSize, quadSize) xRadius:5 yRadius:5];
		if (object == draggedObject)	[[NSColor colorWithCalibratedWhite:0.5 alpha:0.7] set];
		else							[[NSColor colorWithCalibratedWhite:0.3 alpha:0.7] set];
		[path fill];
		[[NSColor colorWithCalibratedWhite:0.6 alpha:1] set];
		[path stroke];
	}
}

- (NSPoint)positionForObject:(DummyObject*)object
{
	return	[object position];
}

- (void)setPosition:(NSPoint)point forObject:(id)object
{
	[object setPosition:point];
}


//
// Drag handling : mouseDown 
//	Record initial position for undo call
//
- (void)mouseDown:(NSEvent *)theEvent
{
	dragStart = [self convertPoint:theEvent.locationInWindow fromView:nil];

	draggedObject = nil;
	for (id object in objects)
	{
		NSPoint p = [self positionForObject:object];
		if (	dragStart.x >= p.x-quadSize/2 && dragStart.x <= p.x+quadSize/2
			&&	dragStart.y >= p.y-quadSize/2 && dragStart.y <= p.y+quadSize/2)
		{
			draggedObject = object;
			dragStartObjectPosition = [self positionForObject:object];
		}
	}
	
	dragging = !!draggedObject;
	dragged = false;

	[self setNeedsDisplay:YES];
}

//
// Drag handling : mouseDragged 
//
- (void)mouseDragged:(NSEvent *)theEvent
{
	if (!dragging)	return;

	NSPoint p = [self convertPoint:theEvent.locationInWindow fromView:nil];
	
	newObjectPosition = NSMakePoint(p.x+dragStartObjectPosition.x-dragStart.x, p.y+dragStartObjectPosition.y-dragStart.y);
	[self setPosition:newObjectPosition forObject:draggedObject];
	
	[self setNeedsDisplay:YES];
	
	dragged = true;
}

//
// Drag handling : mouseup 
//	Set new position by calling an undo friendly method
//
- (void)mouseUp:(NSEvent *)theEvent
{
	if (draggedObject && dragged)
	{
		[self setUndoablePosition:newObjectPosition andOldPosition:dragStartObjectPosition forObject:draggedObject];
		dragging = false;
	}
}

//
// Set an object's new position
//	Record in NSUndoManager the undo call with the original position, saved in mouseDown
//
- (void)setUndoablePosition:(NSPoint)position andOldPosition:(NSPoint)oldPosition forObject:(id)o
{
	NSUndoManager* undoManager = [self undoManager];
	[[undoManager prepareWithInvocationTarget:self] setUndoablePosition:oldPosition andOldPosition:position forObject:o];
	[undoManager setActionName:@"Move"];
	[self setPosition:position forObject:o];

	[self setNeedsDisplay:YES];
	[((UndoWindow*)[self window]) delayDumpUndo];
}




//
// IB Action : new object
//
- (IBAction)add:(id)sender
{
	// Create new object
	id o = [[DummyObject alloc] init];
	[o setWindow:[self window]];

	// Set position
	[self setPosition:NSMakePoint([self frame].size.width/2, [self frame].size.height/2) forObject:o];
	// And add to array (undoable method)
	[self addObject:o];
	// Let only the array keep a reference to the instance : that way the object will be freed when the undo stack is cleared.
	// If we don't do that, it will stay around with a retainCount of 1
	[o release];
}

//
// IB Action : remove selection
//
- (IBAction)remove:(id)sender
{
	if (!draggedObject)	return;

	// Remove from array (undoable method)
	[self removeObject:draggedObject];
}

//
// Add an object to list of objects
//
- (void)addObject:(id)o
{
	// Add to array
	[objects addObject:o];

	// Register converse method : when adding, register remove with the object we were given
	NSUndoManager* undoManager = [self undoManager];
	[undoManager registerUndoWithTarget:self selector:@selector(removeObject:) object:o];

	// Action name displayed in menu
	[undoManager setActionName:@"New"];

	// Set current selection to new object
	draggedObject = o;
	
	// Dump stacks and repaint
	[((UndoWindow*)[self window]) delayDumpUndo];
	[self setNeedsDisplay:YES];
}

//
// Remove an object from list of objects
//
- (void)removeObject:(id)o
{
	// Remove from array
	[objects removeObject:o];

	// Register converse method : when removing, register adding with the object we were given
	NSUndoManager* undoManager = [self undoManager];
	[undoManager registerUndoWithTarget:self selector:@selector(addObject:) object:o];

	// Action name displayed in menu
	[undoManager setActionName:@"Delete"];

	// Clear current selection
	draggedObject = nil;
	
	// Dump stacks and repaint
	[((UndoWindow*)[self window]) delayDumpUndo];
	[self setNeedsDisplay:YES];
}



@end



//
// DummyObject, objects handled and displayed by DummyView 
//
@implementation DummyObject

//
// Store window to call dump method
//
- (void)setWindow:(id)w
{
	window = w;
}

- (void)setPosition:(NSPoint)p
{
	position = p;
}

- (NSPoint)position
{
	return	position;
}

//
// When retaining and releasing, call window's dump method
//
- (id)retain
{
	id o = [super retain];
	
	NSLog(@"+ %x %d", self, [self retainCount]);
	
	[window updateObjectsWithObject:self andRetainCount:[self retainCount]];
	return o;
}

- (oneway void)release
{
	NSLog(@"- %x %d ->%d %@", self, [self retainCount], [self retainCount]-1, [self retainCount]-1 == 0 ? @"*DESTROYED*" : @"");

	[window updateObjectsWithObject:self andRetainCount:[self retainCount]-1];

	return	[super release];
}

@end